unit CaptureForm;
interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons,
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdFTP,
  webcam;

type
  TForm1 = class(TForm)
    btnConnect: TButton;
    Timer1: TTimer;
    IdFTP1: TIdFTP;
    btnGrab: TButton;
    Memo1: TMemo;
    CaptureImage: TImage;
    PreviousImage: TImage;
    cbUpload: TCheckBox;
    btnCapture: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure btnGrabClick(Sender: TObject);
    procedure btnCaptureClick(Sender: TObject);
    procedure Memo1Click(Sender: TObject);
  private
    CaptureWnd: THandle;
    WebCamServiceSoap: TWebCamServiceSoap;
  end;

var
  Form1: TForm1 = nil;

implementation
{$R *.dfm}
uses
  IniFiles, JPeg;

const
  CaptureLeft = 8;
  CaptureTop = 8;
  CaptureWidth = 320;
  CaptureHeight = 240;

function capCreateCaptureWindowA(WindowName: PChar;
  dwStyle: Cardinal; x,y, nWidth,nHeight: Integer;
  ParentWin: HWnd; nID: Integer): HWnd; stdcall
  external 'AVICAP32.DLL';

const
  WM_CAP_DRIVER_CONNECT    = WM_USER + 10;
  WM_CAP_DRIVER_DISCONNECT = WM_USER + 11;
  WM_CAP_SAVEDIB           = WM_USER + 25;
  WM_CAP_GRAB_FRAME        = WM_USER + 60;

const
  BmpFilename = 'g:\www\uk-bug\webcam\'; // webcam.jpg';

const
  MaxDiff = 2000;

procedure TForm1.FormCreate(Sender: TObject);
begin
  CaptureWnd := 0;
  WebCamServiceSoap := GetTWebCamServiceSoap
end;

procedure TForm1.btnConnectClick(Sender: TObject);
const
  str_Connect = 'Connect';
  str_Disconn = 'Disconnect';
begin
  Timer1.Enabled := False;
  if (Sender as TButton).Caption = str_Connect then
  begin
    CaptureWnd := capCreateCaptureWindowA('Dr.Bob''s webcam',
      WS_CHILD or WS_VISIBLE , CaptureLeft, CaptureTop,
      CaptureWidth, CaptureHeight, Handle, 0);
    if CaptureWnd <> 0 then
    begin
      btnGrab.Enabled := True;
      SendMessage(CaptureWnd, WM_CAP_DRIVER_CONNECT, 0, 0);
      SendMessage(CaptureWnd, WM_CAP_SAVEDIB, 0, Cardinal(PChar(BmpFileName + 'savedib.bmp')));
    end;
    (Sender as TButton).Caption := str_Disconn;
  end
  else
  begin
    if CaptureWnd <> 0 then
      SendMessage(CaptureWnd, WM_CAP_DRIVER_DISCONNECT, 0, 0);
    CaptureWnd := 0;
    (Sender as TButton).Caption := str_Connect;
    btnGrab.Enabled := False
  end
end;

procedure TForm1.btnCaptureClick(Sender: TObject);
begin
  if CaptureWnd <> 0 then
    SendMessage(CaptureWnd, WM_CAP_SAVEDIB, 0, Cardinal(PChar('savedib.bmp')))
end;

procedure TForm1.btnGrabClick(Sender: TObject);
{$J+}
const
  LastDateTimeStr: String = '';
type
  TRGB = record
           case Boolean of
             True: (RGBColor: Longint);
             False: (R,G,B: Byte)
           end;
var
  PanelDC: HDC;
  Stream: TFileStream;
  DateTimeStr: String;
//OldScanLine,NewScanLine: PByte; // PByteArray
  OldRGB,NewRGB: TRGB;
  diff,x,y: Integer;
  TimeStamp: Double;
begin
  Timer1.Enabled := False;
  diff := 0;
  ShortDateFormat := 'yyyy-mm-dd';
  if (Time > 7/24) and (Time < 21/24) then if CaptureWnd <> 0 then
  begin
    SendMessage(CaptureWnd, WM_CAP_GRAB_FRAME, 0, 0);
    repeat
      DateTimeStr := DateTimeToStr(Now)
    until DateTimeStr <> LastDateTimeStr; // at least one second delay !!!
    LastDateTimeStr := DateTimeStr;
    Caption := DateTimeStr;
//  SendMessage(CaptureWnd, WM_CAP_SAVEDIB, 0, longint(PChar(BmpFileName)));
    if CaptureImage.Picture.Bitmap.Width <> CaptureWidth then // image doesn't exist, yet
      CaptureImage.Picture.Bitmap := TBitMap.Create;
    try
      CaptureImage.Picture.Bitmap.Width := CaptureWidth;
      CaptureImage.Picture.Bitmap.Height := CaptureHeight;
      PanelDC := GetDC(Handle);
      try
        BitBlt(CaptureImage.Picture.Bitmap.Canvas.Handle, 0, 0,
          CaptureWidth, CaptureHeight, PanelDC, CaptureLeft, CaptureTop, SRCCOPY)
      finally
        ReleaseDC(Handle, PanelDC)
      end;

    { case CaptureImage.Picture.Bitmap.PixelFormat of
       pfDevice: Memo1.Lines.Add('pfDevice');
       pf1bit:   Memo1.Lines.Add('pf1bit');
       pf4bit:   Memo1.Lines.Add('pf4bit');
       pf8bit:   Memo1.Lines.Add('pf8bit');
       pf15bit:  Memo1.Lines.Add('pf15bit');
       pf16bit:  Memo1.Lines.Add('pf16bit');
       pf24bit:  Memo1.Lines.Add('pf24bit');
       pf32bit:  Memo1.Lines.Add('pf32bit');
       pfCustom: Memo1.Lines.Add('pfCustom');
      end; }
      Memo1.Lines.Clear;

      TimeStamp := Now;
      // Motion Detect between Bitmap and Image1.Picture
      diff := 0;
      for y:=0 to CaptureHeight-1 do
      begin
        for x:=0 to CaptureWidth-1 do
        begin
          NewRGB.RGBColor := ColorToRGB(CaptureImage.Picture.Bitmap.Canvas.Pixels[x,y]);
          OldRGB.RGBColor := ColorToRGB(PreviousImage.Picture.Bitmap.Canvas.Pixels[x,y]);
          if (abs(NewRGB.R - OldRGB.R) > $F) and
             (abs(NewRGB.G - OldRGB.G) > $F) and
             (abs(NewRGB.B - OldRGB.B) > $F) then Inc(diff);
        end
      end;
      Memo1.Lines.Add(Format('Difference: %d (in %.2n sec.)',[diff,(Now - TimeStamp) * 24 * 60 * 60]));

      if diff > MaxDiff then // only save when movement detected !!
      with TJPEGImage.Create do
      try
        Assign(CaptureImage.Picture.Bitmap);
        CompressionQuality := 75;
        for x:=Length(DateTimeStr) downto 1 do
          if DateTimeStr[x] in ['-',':'] then Delete(DateTimeStr,x,1)
          else
            if DateTimeStr[x] = ' ' then DateTimeStr[x] := '-';

        Stream := TFileStream.Create(BmpFilename + DateTimeStr + '.jpg',fmCreate);
        try
          SaveToStream(Stream);
          if cbUpload.Checked then
          try
            Stream.Position := 0;
            CompressionQuality := 75;
            Memo1.Lines.Add('Connecting');
            Application.ProcessMessages;
            if not IdFTP1.Connected then
            begin
              IdFTP1.Connect;
              IdFTP1.ChangeDir('webcam');
            end;
            Memo1.Lines.Add('Uploading');
            Application.ProcessMessages;
            IdFTP1.Put(Stream,DateTimeStr+'.jpg');
            WebCamServiceSoap.SetWebCamURL('http://www.bobswart.com/webcam/'+DateTimeStr+'.jpg','geheim');
          except
            on Ex: Exception do
            begin
              Memo1.Lines.Add('Upload failed...');
              IdFTP1.Disconnect // only on error...
            end
          end
        finally
          Stream.Free
        end
      finally
        Free // TJPEGImage
      end;
    finally
      PreviousImage.Picture.Assign(CaptureImage.Picture)
    end
  end;

  with TIniFile.Create(ChangeFileExt(ParamStr(0),'.ini')) do
  try
    Timer1.Interval := ReadInteger('timer','interval',60000);
    if diff > MaxDiff then Timer1.Interval := 100;
    if Timer1.Interval <= 0 then Close // exit
  finally
  end;
  Timer1.Enabled := Timer1.Interval > 0
end;

{
procedure TForm1.btnMovieClick(Sender: TObject);
begin
  if (Sender as TButton).Caption = 'Movie' then
  begin
    if CaptureWnd <> 0 then
    begin
      NextFilename(AviFilename);
      SendMessage(CaptureWnd, WM_CAP_FILE_SET_CAPTURE_FILEA, 0, Longint(PChar(AviFileName)));
      SendMessage(CaptureWnd, WM_CAP_SEQUENCE, 0, 0)
    end;
    (Sender as TButton).Caption := 'End Movie';
  end
  else
  begin
    if CaptureWnd <> 0 then
      SendMessage(CaptureWnd, WM_CAP_STOP, 0, 0);
    (Sender as TButton).Caption := 'Movie'
  end
end;
}

procedure TForm1.Memo1Click(Sender: TObject);
var
  PanelDC: HDC;
  Stream: TFileStream;
begin
  if CaptureWnd <> 0 then
  begin
    SendMessage(CaptureWnd, WM_CAP_GRAB_FRAME, 0, 0);
    if CaptureImage.Picture.Bitmap.Width <> CaptureWidth then // image doesn't exist, yet
      CaptureImage.Picture.Bitmap := TBitMap.Create;
    try
      CaptureImage.Picture.Bitmap.Width := CaptureWidth;
      CaptureImage.Picture.Bitmap.Height := CaptureHeight;
      PanelDC := GetDC(Handle);
      try
        BitBlt(CaptureImage.Picture.Bitmap.Canvas.Handle, 0, 0,
          CaptureWidth, CaptureHeight, PanelDC, CaptureLeft, CaptureTop, SRCCOPY)
      finally
        ReleaseDC(Handle, PanelDC)
      end;

      with TJPEGImage.Create do
      try
        Assign(CaptureImage.Picture.Bitmap);
        CompressionQuality := 100; // local storage
        Stream := TFileStream.Create('webcam42.jpg',fmCreate);
        try
          SaveToStream(Stream);
          if cbUpload.Checked then
          try
            Stream.Position := 0;
            CompressionQuality := 75;
            if not IdFTP1.Connected then IdFTP1.Connect;
            IdFTP1.ChangeDir('www.bobswart.com'); Application.ProcessMessages;
            IdFTP1.ChangeDir('webcam');  Application.ProcessMessages;
            IdFTP1.Put(Stream,'webcam42.jpg');
          except
            on Ex: Exception do
              IdFTP1.Disconnect // only on error...
          end
        finally
          Stream.Free
        end
      finally
        Free // TJPEGImage
      end;
    finally
      PreviousImage.Picture.Assign(CaptureImage.Picture)
    end
  end
end;

initialization
end.
